@@ -329,7 +329,7 @@ class TaskGraph(object):
             ''', self._task_database_path, mode='read_only',
             fetch='one', argument_list=['version'])[0]
         if local_version != __version__:
-            LOGGER.warn(
+            LOGGER.warning(
                 f'the database located at {self._task_database_path} was '
                 f'created with TaskGraph version {local_version} but the '
                 f'current version is {__version__}')
@@ -394,16 +394,16 @@ class TaskGraph(object):
             # it's possible the global state is not well defined, so just in
             # case we'll wrap it all up in a try/except
             self._terminated = True
-            if self._executor_ready_event:
+            if self._executor_ready_event is not None:
                 # alert executors to check that _terminated is True
                 self._executor_ready_event.set()
             LOGGER.debug("shutting down workers")
-            if self._worker_pool:
+            if self._worker_pool is not None:
                 self._worker_pool.close()
                 self._worker_pool.terminate()
                 self._worker_pool = None
 
-            if self._logging_queue:
+            if self._logging_queue is not None:
                 # Close down the logging monitor thread.
                 self._logging_queue.put(None)
                 self._logging_monitor_thread.join(_MAX_TIMEOUT)
@@ -413,6 +413,8 @@ class TaskGraph(object):
                         x = self._logging_queue.get_nowait()
                         LOGGER.debug(
                             "the logging queue had this in it: %s", x)
+                    except queue.Empty:
+                        break
                     except Exception:
                         LOGGER.exception(
                             "Expected an empty logging queue, but if the "
@@ -434,21 +436,28 @@ class TaskGraph(object):
                                 'task executor thread timed out %s',
                                 executor_thread)
                     except Exception:
-                        LOGGER.exception(
-                            "Exception when joining %s", executor_thread)
+                        LOGGER.warning(
+                            f'Joining executor thread {executor_thread} '
+                            f'would have caused a deadlock, skipping.')
                 if self._reporting_interval is not None:
                     LOGGER.debug("joining _monitor_thread.")
                     if self._logging_queue:
                         self._logging_queue.put(None)
                     self._execution_monitor_wait_event.set()
-                    self._execution_monitor_thread.join(_MAX_TIMEOUT)
-                    timedout = self._execution_monitor_thread.is_alive()
-                    if timedout:
-                        LOGGER.debug(
-                            '_monitor_thread %s timed out',
-                            self._execution_monitor_thread)
+                    try:
+                        self._execution_monitor_thread.join(_MAX_TIMEOUT)
+                        timedout = self._execution_monitor_thread.is_alive()
+                        if timedout:
+                            LOGGER.debug(
+                                '_monitor_thread %s timed out',
+                                self._execution_monitor_thread)
+                    except Exception:
+                        LOGGER.warning(
+                            'joining execution monitor thread '
+                            f'{self._execution_monitor_thread} would have '
+                            'caused a deadlock, skipping.')
                     for task in self._task_hash_map.values():
-                        # this is a shortcut to get the tasks to mark as joined
+                        # shortcut to get the tasks to mark as joined
                         task.task_done_executing_event.set()
 
             # drain the task ready queue if there's anything left
@@ -504,7 +513,7 @@ class TaskGraph(object):
                             # there's the possibility for a race condition here
                             # where another thread already closed the worker
                             # pool, so just guard against it
-                            LOGGER.warn('worker pool was already closed')
+                            LOGGER.warning('worker pool was already closed')
                     LOGGER.debug(
                         "no tasks are pending and taskgraph closed, normally "
                         "terminating executor %s." % threading.currentThread())
@@ -627,14 +636,16 @@ class TaskGraph(object):
                 be copied to the new one.
             hardlink_allowed (bool): if ``copy_duplicate_artifact`` is True,
                 this will allow a hardlink rather than a copy when needed.
-            transient_run (bool): if True a call with an identical execution
-                hash will be reexecuted on a subsequent instantiation of a
-                future TaskGraph object. If a duplicate task is submitted
-                to the same object it will not be re-run in any scenario.
-                Otherwise if False, subsequent tasks with an identical
-                execution hash will be skipped.
+            transient_run (bool): if True, this Task will be reexecuted
+                even if it was successfully executed in a previous TaskGraph
+                instance. If False, this Task will be skipped if it was
+                executed successfully in a previous TaskGraph instance. One
+                might wish to set `transient_run` to True on a Task that does
+                some sort of initialization that's needed every time a
+                TaskGraph is instantiated. Perhaps to acquire dynamic resources
+                or authenticate permissions.
             store_result (bool): If True, the result of ``func`` will be stored
-                in the TaskGraph database and retrieveable with a call to
+                in the TaskGraph database and retrievable with a call to
                 ``.get()`` on a ``Task`` object.
 
         Returns:
@@ -1138,7 +1149,7 @@ class Task(object):
 
         """
         LOGGER.debug("_call check if precalculated %s", self.task_name)
-        if self.is_precalculated():
+        if not self._transient_run and self.is_precalculated():
             self.task_done_executing_event.set()
             return
         LOGGER.debug("not precalculated %s", self.task_name)
@@ -1285,12 +1296,12 @@ class Task(object):
             self._target_path_list+self._ignore_path_list,
             self._ignore_directories))
 
-        other_arguments = list(_filter_non_files(
+        other_arguments = _filter_non_files(
             [self._reexecution_info['args_clean'],
              self._reexecution_info['kwargs_clean']],
             self._target_path_list,
             self._ignore_path_list,
-            self._ignore_directories))
+            self._ignore_directories)
 
         LOGGER.debug("file_stat_list: %s", file_stat_list)
         LOGGER.debug("other_arguments: %s", other_arguments)
@@ -1425,8 +1436,7 @@ def _get_file_stats(
 
     Args:
         base_value: any python value. Any file paths in ``base_value``
-            should be "os.path.norm"ed before this function is called.
-            contains filepaths in any nested structure.
+            should be processed with `_normalize_path`.
         hash_algorithm (string): either a hash function id that
             exists in hashlib.algorithms_available, 'exists', or
             'sizetimestamp'. Any paths to actual files in the arguments will be
@@ -1500,17 +1510,20 @@ def _filter_non_files(
             out.
 
     Return:
-        original``base_value`` with any nested file paths for files that
-        exist in the os.exists removed.
+        original ``base_value`` with any nested file paths for files that
+        exist in the os.exists set to ``None``.
 
     """
     if isinstance(base_value, _VALID_PATH_TYPES):
         try:
             norm_path = _normalize_path(base_value)
-            if norm_path not in ignore_list and (norm_path in keep_list or (
-                    os.path.isdir(norm_path) and keep_directories) or
-                    not os.path.isfile(norm_path)):
-                yield norm_path
+            if norm_path not in ignore_list and (
+                    norm_path in keep_list or ((
+                        os.path.isdir(norm_path) and keep_directories) or (
+                        not os.path.isfile(norm_path) and
+                        not os.path.isdir(norm_path)))):
+                return norm_path
+            return None
         except (OSError, ValueError):
             # I ran across a ValueError when one of the os.path functions
             # interpreted the value as a path that was too long.
@@ -1520,18 +1533,18 @@ def _filter_non_files(
             LOGGER.exception(
                 "base_value couldn't be analyzed somehow '%s'", base_value)
     elif isinstance(base_value, dict):
-        for key in base_value.keys():
-            value = base_value[key]
-            for filter_value in _filter_non_files(
-                    value, keep_list, ignore_list, keep_directories):
-                yield (value, filter_value)
+        return {
+            key: _filter_non_files(
+                value, keep_list, ignore_list, keep_directories)
+            for key, value in base_value.items()
+        }
     elif isinstance(base_value, (list, set, tuple)):
-        for value in base_value:
-            for filter_value in _filter_non_files(
-                    value, keep_list, ignore_list, keep_directories):
-                yield filter_value
+        return type(base_value)([
+            _filter_non_files(
+                value, keep_list, ignore_list, keep_directories)
+            for value in base_value])
     else:
-        yield base_value
+        return base_value
 
 
 def _scrub_task_args(base_value, target_path_list):
@@ -1634,15 +1647,15 @@ def _normalize_path(path):
     except TypeError:
         # this occurs when encountering VERY long strings that might be
         # interpreted as paths
-        LOGGER.warn(
+        LOGGER.warning(
             "failed to abspath %s so returning normalized path instead")
         abs_path = norm_path
     return os.path.normcase(abs_path)
 
 
 @retrying.retry(
-    wait_exponential_multiplier=100, wait_exponential_max=3200,
-    stop_max_attempt_number=5)
+    wait_exponential_multiplier=500, wait_exponential_max=3200,
+    stop_max_attempt_number=100)
 def _execute_sqlite(
         sqlite_command, database_path, argument_list=None,
         mode='read_only', execute='execute', fetch=None):
@@ -1701,12 +1714,20 @@ def _execute_sqlite(
         cursor.close()
         connection.commit()
         connection.close()
+        cursor = None
+        connection = None
         return result
+    except sqlite3.OperationalError:
+        LOGGER.warning(
+            'TaskGraph database is locked because another process is using '
+            'it, waiting for a bit of time to try again')
+        raise
     except Exception:
         LOGGER.exception('Exception on _execute_sqlite: %s', sqlite_command)
+        raise
+    finally:
         if cursor is not None:
             cursor.close()
         if connection is not None:
             connection.commit()
             connection.close()
-        raise
