@@ -5,6 +5,7 @@ import logging.handlers
 import multiprocessing
 import os
 import pickle
+import pathlib
 import re
 import shutil
 import sqlite3
@@ -22,7 +23,7 @@ MAX_TRY_WAIT_MS = 500
 
 
 def _return_value_once(value):
-    """Returns the value passed to it only once."""
+    """Return the value passed to it only once."""
     if hasattr(_return_value_once, 'executed'):
         raise RuntimeError("this function was called twice")
     _return_value_once.executed = True
@@ -30,7 +31,7 @@ def _return_value_once(value):
 
 
 def _noop_function(**kwargs):
-    """Does nothing except allow kwargs to be passed."""
+    """Do nothing except allow kwargs to be passed."""
     pass
 
 
@@ -603,7 +604,6 @@ class TaskGraphTests(unittest.TestCase):
         _ = task_graph.add_task(
             func=_create_list_on_disk,
             args=(value, list_len),
-            transient_run=True,
             kwargs={
                 'target_path': target_path,
             })
@@ -617,6 +617,7 @@ class TaskGraphTests(unittest.TestCase):
         _ = task_graph2.add_task(
             func=_create_list_on_disk,
             args=(value, list_len),
+            transient_run=True,
             kwargs={
                 'target_path': target_path,
             })
@@ -1365,22 +1366,25 @@ class TaskGraphTests(unittest.TestCase):
 
     def test_return_value(self):
         """TaskGraph: test that ``.get`` behavior works as expected."""
+        if hasattr(_return_value_once, 'executed'):
+            del _return_value_once.executed
         n_iterations = 3
+        task_graph = taskgraph.TaskGraph(self.workspace_dir, 0, 0)
         for iteration_id in range(n_iterations):
             transient_run = iteration_id == n_iterations-1
             LOGGER.debug(iteration_id)
-            task_graph = taskgraph.TaskGraph(self.workspace_dir, 0, 0)
             expected_value = 'a good value'
             value_task = task_graph.add_task(
                 func=_return_value_once,
                 transient_run=transient_run,
                 store_result=True,
-                args=(expected_value,))
+                args=(expected_value,),
+                task_name=f'{expected_value} iter {iteration_id}')
             value = value_task.get()
             self.assertEqual(value, expected_value)
-            task_graph.close()
-            task_graph.join()
-            task_graph = None
+        task_graph.close()
+        task_graph.join()
+        task_graph = None
 
         # reset run
         del _return_value_once.executed
@@ -1393,7 +1397,8 @@ class TaskGraphTests(unittest.TestCase):
                     func=_return_value_once,
                     transient_run=True,
                     store_result=True,
-                    args=(expected_value,))
+                    args=(expected_value,),
+                    task_name=f'first re-run transient')
                 value = value_task.get()
                 self.assertEqual(value, expected_value)
             else:
@@ -1402,7 +1407,8 @@ class TaskGraphTests(unittest.TestCase):
                         func=_return_value_once,
                         transient_run=True,
                         store_result=True,
-                        args=(expected_value,))
+                        args=(expected_value,),
+                        task_name=f'expected error {iteration_id}')
                     value = value_task.get()
 
             task_graph.close()
@@ -1489,6 +1495,85 @@ class TaskGraphTests(unittest.TestCase):
         task_graph._execution_monitor_thread.join(5)
         self.assertFalse(task_graph._execution_monitor_thread.is_alive())
 
+    def test_dictionary_arguments(self):
+        """TaskGraph: test that large dictionary arguments behave well."""
+        task_graph = taskgraph.TaskGraph(self.workspace_dir, -1)
+        dict_arg = {}
+        x = {None: None}
+        for _ in range(10000):
+            dict_arg[_] = x
+
+        def my_op(dict_arg):
+            pass
+        task_graph.add_task(
+            func=my_op, args=(), kwargs={'dict_arg': dict_arg})
+        task_graph.join()
+        self.assertTrue(True, 'no memory error so everything is fine')
+
+    def test_filter_non_files(self):
+        """TaskGraph: test internal filter non-files function."""
+        from taskgraph.Task import _filter_non_files
+        from taskgraph.Task import _normalize_path
+
+        # Test a passthrough
+        test_dict = {
+            0: {'one': 0, 'two': 1, 'three': 2},
+            1: {'one': 1, 'two': 2, 'three': 3},
+            2: {'one': 2, 'two': 3, 'three': 4}}
+        self.assertEqual(
+            test_dict, _filter_non_files(test_dict, [], [], False))
+
+        # Test combination of files, not existing files, and flags in the
+        # call
+        test_file_a_exists = _normalize_path(os.path.join(
+            self.workspace_dir, 'exists_a.txt'))
+        pathlib.Path(test_file_a_exists).touch()
+        test_file_b_exists = _normalize_path(os.path.join(
+            self.workspace_dir, 'exists_b.txt'))
+        pathlib.Path(test_file_b_exists).touch()
+        test_file_not_a_exists = _normalize_path(os.path.join(
+            self.workspace_dir, 'does_not_exist_a.txt'))
+        test_file_not_b_exists = _normalize_path(os.path.join(
+            self.workspace_dir, 'does_not_exist_b.txt'))
+
+        test_dict = {
+            0: {'one': 0, 'two': 1, 'three': 2},
+            1: {'one': 1, 'two': 2, 'three': 3},
+            2: {'one': 2, 'two': 3, 'three': 4},
+            4: {'bar': test_file_not_a_exists},
+            5: {'foo': test_file_a_exists},
+            6: test_file_b_exists,
+            7: test_file_not_b_exists,
+            8: _normalize_path(self.workspace_dir)}
+
+        expected_result_dict = {
+            0: {'one': 0, 'two': 1, 'three': 2},
+            1: {'one': 1, 'two': 2, 'three': 3},
+            2: {'one': 2, 'two': 3, 'three': 4},
+            4: {'bar': test_file_not_a_exists},
+            5: {'foo': None},
+            6: test_file_b_exists,
+            7: None,
+            8: _normalize_path(self.workspace_dir)}
+
+        self.assertEqual(
+            _filter_non_files(
+                test_dict,
+                [test_file_b_exists],
+                [test_file_not_b_exists],
+                True),
+            expected_result_dict)
+
+        # and test same as above but don't keep directories:
+        expected_result_dict[8] = None
+        self.assertEqual(
+            _filter_non_files(
+                test_dict,
+                [test_file_b_exists],
+                [test_file_not_b_exists],
+                False),
+            expected_result_dict)
+
 
 def Fail(n_tries, result_path):
     """Create a function that fails after ``n_tries``."""
